/*
  
  GUI fuer meinen Rasberry PI...
  ==============================
        Uwe Berger; 2014
 
  --> Abfrage meiner Wetter-Datenbank
  --> rudimentäre Steuerung MPD (vorhandene Playliste)
 
  ---------
  Have fun! 
 
*/



#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <time.h>
#include <dirent.h>
#include <fnmatch.h>
#include <sys/stat.h>
#include <sqlite3.h>
#include <tslib.h>
#include <libmpd/libmpd.h>

#ifdef MPD_DEBUG
#include <libmpd/debug_printf.h>
#endif

#include "framebuffer.h"

// ein paar Systemkommandos verpacken...
// ...System anhalten/ausschalten
#define SHUTDOWN		system("sudo shutdown -Ph now")
// ...Display-Backlight aus-/einschalten
#define BACKLIGHT_SWITCH_OFF system("echo 1 | sudo tee /sys/class/backlight/*/bl_power 1>/dev/null")
#define BACKLIGHT_SWITCH_ON  system("echo 0 | sudo tee /sys/class/backlight/*/bl_power 1>/dev/null")

// Definitionen Bedienknoepfe
#define NR_BUTTONS 			15
#define BUTTON_EXIT			0
#define BUTTON_WEATHER		1
#define BUTTON_MUSIC		2
#define BUTTON_QUEUE		3
#define BUTTON_PREV			4
#define BUTTON_PLAY			5
#define BUTTON_NEXT			6
#define BUTTON_STOP			7
#define BUTTON_REPEAT		8
#define BUTTON_RANDOM		9
#define BUTTON_PL_PREV		10
#define BUTTON_PL_NEXT		11
#define BUTTON_PL_ADD		12
#define BUTTON_PL_REPLACE	13
#define BUTTON_PL_RELOAD    14

#define MODE_EXIT		BUTTON_EXIT
#define MODE_WEATHER	BUTTON_WEATHER
#define MODE_MUSIC		BUTTON_MUSIC
#define MODE_QUEUE		BUTTON_QUEUE

#define BUTTON_SWITCH_ON(b)   buttons[b].flags |=  BUTTON_ON
#define BUTTON_SWITCH_OFF(b)  buttons[b].flags &= ~BUTTON_ON

#define BUTTON_IS_VISIBLE(b)  buttons[b].flags |=  BUTTON_VISIBLE
#define BUTTON_NOT_VISIBLE(b) buttons[b].flags &= ~BUTTON_VISIBLE

static button_t buttons[NR_BUTTONS] = {
// "feste" Button	
{270, 219, 49, 20, "Quit",    WHITE, BLUE,  YELLOW, FONT_8X16, WHITE, YELLOW, BLUE, WHITE, RED, YELLOW, 2},	
{0  , 219, 86, 20, "Weather", WHITE, BLUE,  YELLOW, FONT_8X16, WHITE, YELLOW, BLUE, WHITE, RED, YELLOW, 2},	
{90 , 219, 86, 20, "Music",   WHITE, BLUE,  YELLOW, FONT_8X16, WHITE, YELLOW, BLUE, WHITE, RED, YELLOW, 2},		
{180, 219, 86, 20, "Queue",   WHITE, BLUE,  YELLOW, FONT_8X16, WHITE, YELLOW, BLUE, WHITE, RED, YELLOW, 2},
// Screen "Music"
{5 , 190, 49, 20,  "Prev",   WHITE, GREEN, WHITE,  FONT_8X16, WHITE, YELLOW, BLUE, WHITE, RED, YELLOW, 0},
{57 , 190, 49, 20, "Play",   WHITE, GREEN, WHITE,  FONT_8X16, WHITE, YELLOW, BLUE, WHITE, RED, YELLOW, 0},
{109, 190, 49, 20, "Next",   WHITE, GREEN, WHITE,  FONT_8X16, WHITE, YELLOW, BLUE, WHITE, RED, YELLOW, 0},
{161, 190, 49, 20, "Stop",   WHITE, GREEN, WHITE,  FONT_8X16, WHITE, YELLOW, BLUE, WHITE, RED, YELLOW, 0},
{213, 190, 49, 20, "Repeat", WHITE, GREEN, WHITE,  FONT_8X16, WHITE, YELLOW, BLUE, WHITE, RED, YELLOW, 0},
{265, 190, 49, 20, "Random", WHITE, GREEN, WHITE,  FONT_8X16, WHITE, YELLOW, BLUE, WHITE, RED, YELLOW, 0},
// Screen "Queue"
{  5, 70, 74, 20, "Prev",    WHITE, GREEN, WHITE,  FONT_8X16, WHITE, YELLOW, BLUE, WHITE, RED, YELLOW, 0},
{239, 70, 74, 20, "Next",    WHITE, GREEN, WHITE,  FONT_8X16, WHITE, YELLOW, BLUE, WHITE, RED, YELLOW, 0},
{ 83, 70, 74, 20, "Add",     WHITE, GREEN, WHITE,  FONT_8X16, WHITE, YELLOW, BLUE, WHITE, RED, YELLOW, 0},
{161, 70, 74, 20, "Replace", WHITE, GREEN, WHITE,  FONT_8X16, WHITE, YELLOW, BLUE, WHITE, RED, YELLOW, 0},
{  5, 94, 308, 20, "Playlists reload", WHITE, GREEN, WHITE,  FONT_8X16, WHITE, YELLOW, BLUE, WHITE, RED, YELLOW, 0}
};

// Backlight
#define BACKLIGHT_TIME	60					// ...in Sekunden
unsigned int backlight_timer = BACKLIGHT_TIME;
unsigned char backlight_on = 1;

// Definitionen fuer Wetterdaten
#define WEATHER_LOG_DB 		"/media/fritzbox/WD-MyPassport0810-01/dockstar/wetter_db/weather_log_db"
#define BACKUP_PATH 		"/media/fritzbox/WD-MyPassport0810-01/dockstar/backup"
#define BACKUP_FILE_PATTERN "weather_log_db.*.gz"
#define ALTITUDE	47.5
#define MSG_NONE	0
#define MSG_HINT	1
#define MSG_ERROR	2
char wdb_last_backup[40];
char wdb_temperature_out[40] = "";
char wdb_temperature_in[40]  = "";
char wdb_humidity_hih[40]    = "";
char wdb_humidity_sht[40]    = "";
char wdb_pressure_abs[40]    = "";
char wdb_pressure_rel[40]    = "";
char wdb_voltage_radio[40]   = "";
char wdb_msg[200]            = "";

// Definitionen fuer MPD
#define MAX_LEN_STRING	37
MpdObj *mpd = NULL;
char *mpd_hostname = "localhost";
int   mpd_port     = 6600;
char *mpd_password = "";
char  mpd_artist[MAX_LEN_STRING] = "";
char  mpd_album[MAX_LEN_STRING] = "";
char  mpd_title[MAX_LEN_STRING] = "";
char  mpd_track[MAX_LEN_STRING] = "";
char  mpd_name[MAX_LEN_STRING] = "";
char  mpd_comment[MAX_LEN_STRING] = "";
//char  mpd_state[10] = "";
char  mpd_total_time_str[10] = "";
char  mpd_elapsed_time_str[10] = "";
char  mpd_playlist_str[MAX_LEN_STRING] = "";
int   mpd_total_time_int = 0;
int   mpd_elapsed_time_int = 0;
int   mpd_playlist_len_int = 0;
int   mpd_playlist_pos_int = 0;
char  mpd_playlist_pos_str[10] = "";
char  mpd_playlist_len_str[10] = "";
char  mpd_format_str[50] = "No formatinfo!";
int	  mpd_status_play = 0;

static int  xres, yres;

time_t now;
struct tm *now_tm;
char date_time[21] = "";

struct playlist_struct {
	char name[30];
	struct playlist_struct* prev;
	struct playlist_struct* next;
};

struct playlist_struct* playlists = NULL;

int mode = MODE_MUSIC;

// forwards
void mpd_init(void);
void mpd_error_callback(MpdObj *mi,int errorid, char *msg, void *userdata);
void mpd_status_changed(MpdObj *mi, ChangedStatusType what);
void mpd_lsplaylists(MpdObj *mi);

//**********************************************************************
void mpd_init(void)
{
	mpd = mpd_new(mpd_hostname, mpd_port, mpd_password);
	mpd_signal_connect_error(mpd,(ErrorCallback)mpd_error_callback, NULL);
    mpd_signal_connect_status_changed(mpd,(StatusChangedCallback)mpd_status_changed, NULL);
	mpd_set_connection_timeout(mpd, 10);
	mpd_connect(mpd);
}

//**********************************************************************
void mpd_lsplaylists(MpdObj *mi) 
{
	MpdData *data;
	struct playlist_struct* ptr;
	struct playlist_struct* prev;

	// eventuell alte vorhandene Liste mit Playlisten loeschen
	if (playlists != NULL) {
		// an Anfang der Liste...
		while (playlists->prev!=NULL) playlists=playlists->prev;
		// ...und bis zum Ende loeschen
		while (playlists != NULL) {
			ptr = playlists->next;
			free(playlists);
			playlists = ptr;	
		}
	}	
	// Liste mit Playlisten erzeugen...
	data=mpd_database_playlist_list(mi);
	for (; data; data=mpd_data_get_next(data)){
		if (data->type ==  MPD_DATA_TYPE_PLAYLIST) {
			if (playlists == NULL) {
				playlists = malloc(sizeof(*playlists));	
				strncpy(playlists->name, data->playlist->path,29);
				playlists->prev = NULL;
				playlists->next = NULL;
			} else {
				ptr = playlists;
				while (ptr->next != NULL) ptr = ptr->next;
				prev = ptr;
				ptr->next = malloc(sizeof(*ptr));	
				ptr=ptr->next;
				strncpy(ptr->name, data->playlist->path,29);
				ptr->prev = prev;
				ptr->next = NULL;
			}	
		}
	}
}

//**********************************************************************
void mpd_error_callback(MpdObj *mi,int errorid, char *msg, void *userdata)
{
	printf("mpd-Error %i: '%s'\n", errorid, msg);
	switch (errorid) {
		case 10:
			mpd_init();
			mpd_lsplaylists(mpd);
			break;
	}
} 

//**********************************************************************
void mpd_status_changed(MpdObj *mi, ChangedStatusType what)
{
	//printf("%i\n", what);
	// aktuelle Playliste hat sich geaendert
	//printf("mpd_status_changed\n");
	
	if(what&MPD_CST_PLAYLIST) {
		mpd_playlist_len_int = mpd_playlist_get_playlist_length(mi);
		sprintf(mpd_playlist_str, "Position: %i/%i", mpd_playlist_pos_int, mpd_playlist_len_int);
		sprintf(mpd_playlist_len_str, "%i", mpd_playlist_len_int);
	}
	// aktueller Song hat sich geaendert
	if((what&MPD_CST_SONGID)||(what&MPD_CST_PLAYLIST)) {
		mpd_Song *song = mpd_playlist_get_current_song(mi);
		if(song) {
			mpd_song_markup(mpd_artist, sizeof(mpd_artist), "%artist%", song);
			mpd_song_markup(mpd_album, sizeof(mpd_album), "%album%", song);
			mpd_song_markup(mpd_title, sizeof(mpd_title), "%title%", song);
			mpd_song_markup(mpd_track, sizeof(mpd_track), "%track%", song);
			mpd_song_markup(mpd_name, sizeof(mpd_name), "%name%", song);
			mpd_song_markup(mpd_comment, sizeof(mpd_comment), "%comment%", song);
			mpd_playlist_pos_int = song->pos + 1;
			sprintf(mpd_playlist_str, "Position: %i/%i", mpd_playlist_pos_int, mpd_playlist_len_int);
			sprintf(mpd_playlist_pos_str, "%i", mpd_playlist_pos_int);
						
/*
			printf("artist:  %s\n", mpd_artist);
			printf("album:   %s\n", mpd_album);
			printf("title:   %s\n", mpd_title);
			printf("track:   %s\n", mpd_track);
			printf("name:    %s\n", mpd_name);
			printf("comment: %s\n", mpd_comment);
			printf("----------\n");
*/
		}
	}
	// Abspielstatus hat sich geaendert
	if(what&MPD_CST_STATE) {
		switch(mpd_player_get_state(mi)) {
			case MPD_PLAYER_PLAY:
				BUTTON_SWITCH_ON(BUTTON_PLAY);
				mpd_status_play = 1;
				backlight_timer = BACKLIGHT_TIME;
				if (!backlight_on) {
					backlight_on = 1;
					BACKLIGHT_SWITCH_ON;
				}
				if (mode != MODE_QUEUE) mode = MODE_MUSIC;
				break;
			case MPD_PLAYER_PAUSE:
			case MPD_PLAYER_STOP:
				BUTTON_SWITCH_OFF(BUTTON_PLAY);
				mpd_status_play = 0;
				break;
			default:
				break;
		}
	}
	// Gesamtabspielzeit des aktuellen Songs hat sich geaendert
	if(what&MPD_CST_TOTAL_TIME){
		mpd_total_time_int = mpd_status_get_total_song_time(mi);
		sprintf(mpd_total_time_str, "%02i:%02i", 
				mpd_total_time_int/60, mpd_total_time_int%60);
	}
	// vergangene Abspielzeit des aktuellen Songs hat sich geaendert
	if(what&MPD_CST_ELAPSED_TIME){
		mpd_elapsed_time_int = mpd_status_get_elapsed_song_time(mi);
		sprintf(mpd_elapsed_time_str, "%02i:%02i", 
				mpd_elapsed_time_int/60, mpd_elapsed_time_int%60);
	}
	// Wiederholung?
	if(what&MPD_CST_REPEAT){
		if (mpd_player_get_repeat(mi)) {
			BUTTON_SWITCH_ON(BUTTON_REPEAT);
		} else {
			BUTTON_SWITCH_OFF(BUTTON_REPEAT);
		}
	}
	// Zufallswiedergabe?
	if(what&MPD_CST_RANDOM){
		if (mpd_player_get_random(mi)) {
			BUTTON_SWITCH_ON(BUTTON_RANDOM);
		} else {
			BUTTON_SWITCH_OFF(BUTTON_RANDOM);
		}
	}
/*	
	sprintf(mpd_format_str, "%i Can./%ibps/%ikbs/%iHz",
				mpd_status_get_channels(mi),
				mpd_status_get_bits(mi),				
				mpd_status_get_bitrate(mi),
				mpd_status_get_samplerate(mi));			
*/
}


//**********************************************************************
void weather_db_read(void) 
{
	DIR *d;
	struct dirent *d_entry;
	struct stat s;
	time_t f_mtime;
	struct tm *f_tm;	
	char entry_path[PATH_MAX + 1];
	size_t path_len;
	
	sqlite3 *db;
	char *sql;
	sqlite3_stmt *vm;
	double current_pressure;
	
	// letztes Backup?
	strncpy (entry_path, BACKUP_PATH, sizeof (entry_path));
	path_len = strlen(BACKUP_PATH);
	if (entry_path[path_len - 1] != '/') {
    	entry_path[path_len] = '/';
    	entry_path[path_len + 1] = '\0';
    	++path_len;
  	}
	if ((d=opendir(BACKUP_PATH)) == NULL) {
		sprintf(wdb_msg, "Error %s", "can't open backup-directory!");
		return;		
	}
	f_mtime = 0;
	while ((d_entry = readdir(d)) != NULL) {
   		strncpy (entry_path + path_len, d_entry->d_name, sizeof (entry_path) - path_len);
   		stat(entry_path, &s);
		if ((fnmatch(BACKUP_FILE_PATTERN, d_entry->d_name, 0) == 0) &&
		 	(s.st_mtime > f_mtime)) {
			f_mtime = s.st_mtime;
			f_tm = localtime (&f_mtime);	
		}
 	} 
	if (f_mtime) {
		sprintf(wdb_last_backup, "last backup    : %02d.%02d.%04d; %02d:%02d:%02d", 
					f_tm->tm_mday, f_tm->tm_mon+1, f_tm->tm_year+1900,
					f_tm->tm_hour, f_tm->tm_min, f_tm->tm_sec);
	} else {
		sprintf(wdb_last_backup, "last backup    : %s", "unknown!");		
	}
	closedir(d);
	
	// Wetterdaten aus sqlite-DB einlesen
	// Datenbank oeffnen
	if(sqlite3_open(WEATHER_LOG_DB, &db)){
		sprintf(wdb_msg, "Error %s", sqlite3_errmsg(db));
		sqlite3_close(db);
		return;
	} else {
		sprintf(wdb_msg, "%s", "Database is ready...");
	}
	// Aussen-/Innentemperatur ausgeben
	sql = "select sht15_temp, last_values.'107D6EB501080057' from last_values where tab = 'temp_log'";
	if (sqlite3_prepare(db, sql, -1, &vm, NULL)) {
		sprintf(wdb_msg, "Error %s", sqlite3_errmsg(db));
		sqlite3_close(db);
		return;
	} else {
	while(sqlite3_step(vm) != SQLITE_DONE){
			sprintf(wdb_temperature_out, "temp. (out)    : %6.1f%cC", sqlite3_column_double(vm, 0), 248);
			sprintf(wdb_temperature_in,  "temp. (in)     : %6.1f%cC", sqlite3_column_double(vm, 1), 248);
		}
	}
	sqlite3_finalize(vm);	
	// Luftfeuchtigkeit HIH-4020, SHT-15 ausgeben
	sql = "select hih4030_hum, sht15_hum from last_values where tab = 'humidity_log'";
	if (sqlite3_prepare(db, sql, -1, &vm, NULL)) {
		sprintf(wdb_msg, "Error %s", sqlite3_errmsg(db));
		sqlite3_close(db);
		return;
	} else {
		while(sqlite3_step(vm) != SQLITE_DONE){
			sprintf(wdb_humidity_hih, "humidity (HIH) : %6.1f%%", sqlite3_column_double(vm, 0));
			sprintf(wdb_humidity_sht, "humidity (SHT) : %6.1f%%", sqlite3_column_double(vm, 1));
		}
	}
	sqlite3_finalize(vm);	
	// relativer/absoluten Luftdruck ausgeben
	sql = sqlite3_mprintf("select pressure, pressure+%f/8.0 from last_values where tab = 'pressure_log'", ALTITUDE);
	if (sqlite3_prepare(db, sql, -1, &vm, NULL)) {
		sprintf(wdb_msg, "Error %s", sqlite3_errmsg(db));
		sqlite3_close(db);
		return;
	} else {
		while(sqlite3_step(vm) != SQLITE_DONE){
			//sprintf(wdb_pressure_abs, "pressure (abs.): %6.1fhPa", sqlite3_column_double(vm, 0));
			sprintf(wdb_pressure_rel, "pressure (rel.): %6.1fhPa", sqlite3_column_double(vm, 1));
			current_pressure=sqlite3_column_double(vm, 0);
		}
	}
	sqlite3_finalize(vm);
	// Tendenz Luftdruck ausgeben
	sql = "select pressure from last_values where tab = 'pressure_3h_avg'";
	if (sqlite3_prepare(db, sql, -1, &vm, NULL)) {
		sprintf(wdb_msg, "Error %s", sqlite3_errmsg(db));
		sqlite3_close(db);
		return;
	} else {
		while(sqlite3_step(vm) != SQLITE_DONE){
			if (current_pressure > sqlite3_column_double(vm, 0)) {
				sprintf(wdb_pressure_abs, "pressure (abs.): %6.1fhPa %c", current_pressure, 24);
			} else {
				sprintf(wdb_pressure_abs, "pressure (abs.): %6.1fhPa %c", current_pressure, 25);
			}
		}
	}
	sqlite3_finalize(vm);
	// Versorgungsspannung Funkmodul ausgeben
	sql = "select rfm12_voltage from last_values where tab = 'voltage_log'";
	if (sqlite3_prepare(db, sql, -1, &vm, NULL)) {
		sprintf(wdb_msg, "Error %s", sqlite3_errmsg(db));
		sqlite3_close(db);
		return;
	} else {
	while(sqlite3_step(vm) != SQLITE_DONE){
			sprintf(wdb_voltage_radio, "voltage (radio): %6.2fV", sqlite3_column_double(vm, 0));
		}
	}
	sqlite3_finalize(vm);	
	// Datenbank schliessen
	sqlite3_close(db);
}


// ********************************************
void weather_db_output(void)
{
	int y = 30;
	int dy = 20;
	// Inhalt
	put_string(10, y, wdb_temperature_out, LIGHT_GREY, FONT_8X16);
	y+=dy;
	put_string(10, y, wdb_temperature_in,  LIGHT_GREY, FONT_8X16);
	y+=dy;
	put_string(10, y, wdb_humidity_hih,    LIGHT_GREY, FONT_8X16);
	y+=dy;
	put_string(10, y, wdb_humidity_sht,    LIGHT_GREY, FONT_8X16);
	y+=dy;
	put_string(10, y, wdb_pressure_abs,    LIGHT_GREY, FONT_8X16);
	y+=dy;
	put_string(10, y, wdb_pressure_rel,    LIGHT_GREY, FONT_8X16);
	y+=dy;
	put_string(10, y, wdb_voltage_radio,   LIGHT_GREY, FONT_8X16);
	y+=dy;
	put_string(10, y, wdb_last_backup,     LIGHT_GREY, FONT_8X16);
	y+=dy;
	put_string(10, y, wdb_msg,		       GREY,       FONT_8X8);
}

// ********************************************
void font_output(void)
{
	int i=0; 
	int x, y;
	char str[2];
	for (y=0; y<8; y++) 
		for (x=0; x<32; x++) {
			sprintf(str, "%c", i);
			put_string(20+x*font[FONT_8X16].width, 50+y*font[FONT_8X16].height, str, GREY, FONT_8X16);
			i++;
		}
}

// ********************************************
void mpd_music_output(void)
{
	int y = 20;
	int dy = 20;	
	int dy1 = 10;
	int x1, x2, dx;
	int no_info = 1;

	if (strlen(mpd_artist)) {
		put_string(10, y, "Artist:", GREY, FONT_8X8);
		y+=dy1;
		put_string(10, y, mpd_artist, LIGHT_GREY, FONT_8X16);
		y+=dy;
		no_info=0;
	} else {
		if (strlen(mpd_name)) {
			put_string(10, y, "Name:", GREY, FONT_8X8);
			y+=dy1;
			put_string(10, y, mpd_name, LIGHT_GREY, FONT_8X16);
			y+=dy;		
			no_info=0;
		}
	}
			
	if (strlen(mpd_album)) {
		put_string(10, y, "Album:", GREY, FONT_8X8);
		y+=dy1;
		put_string(10, y, mpd_album, LIGHT_GREY, FONT_8X16);
		y+=dy;
		no_info=0;
	}
	
	if (strlen(mpd_title)) {
		put_string(10, y, "Title:", GREY, FONT_8X8);
		y+=dy1;
		put_string(10, y, mpd_title, LIGHT_GREY, FONT_8X16);
		y+=dy;
		no_info=0;
	}
	
	// Zeitbalken...
	if (mpd_total_time_int) {
		put_string(10, y, "Songtime:", GREY, FONT_8X8);
		y+=dy1;
		put_string(10, y, mpd_elapsed_time_str, LIGHT_GREY, FONT_8X16);
		put_string(xres-10-8*5, y, mpd_total_time_str, LIGHT_GREY, FONT_8X16);
		x1=10+8*5+5;
		x2=xres-10-8*5-5;
		dx=x2-x1;
		draw_rect(x1, y, x1+dx, y+16, LIGHT_GREY);
		if (mpd_total_time_int) dx=dx*mpd_elapsed_time_int/mpd_total_time_int; else dx=0;
		draw_fillrect(x1, y, x1+dx, y+16, LIGHT_GREY);
		y+=dy;
		no_info=0;
	}

	if (no_info) put_string_center(160, 80, "No Info!", LIGHT_GREY, FONT_8X16);

	y+=150;
	put_string(10, y, "Current queue:", GREY, FONT_8X8);
	y+=dy1;
	put_string(10, y, mpd_playlist_str, LIGHT_GREY, FONT_8X16);

	y+=dy;
	put_string(10, y, mpd_format_str, LIGHT_GREY, FONT_8X8);
}

// ********************************************
void mpd_queue_output(void)
{
	int y = 30;
	
	if (playlists != NULL) {
		put_string(10, y, "Playlists:", GREY, FONT_8X8);
		y+=20;
		put_string(10, y, playlists->name, LIGHT_GREY, FONT_8X16);
	} else {
		put_string_center(160, 80, "No playlists found!", LIGHT_GREY, FONT_8X16);
	}
}

// ********************************************
static void sig(int sig)
{
	close_framebuffer();
	fflush(stderr);
	fflush(stdout);
	BACKLIGHT_SWITCH_ON;
	exit(1);
}

// ********************************************
	void refresh_screen(void)
{
	int i;
	
	clear_screen();
	// Rahmen
	draw_rect(0, font[FONT_8X16].height/2, xres-1, 215, WHITE);
	// Datum/Uhrzeit
	draw_fillrect(xres-(strlen(date_time)+1)*font[FONT_8X16].width-2, 0,
	              xres-font[FONT_8X16].width, font[FONT_8X16].height/2,
	              BLACK);
	put_string(xres-(strlen(date_time)+1)*font[FONT_8X16].width, 0, date_time, WHITE, FONT_8X16);
	// Status "fester" Button zuruecksetzen
	BUTTON_SWITCH_OFF(BUTTON_WEATHER);
	BUTTON_SWITCH_OFF(BUTTON_MUSIC);
	BUTTON_SWITCH_OFF(BUTTON_QUEUE);
	// alle "variablen" Button unsichtbar
	BUTTON_NOT_VISIBLE(BUTTON_PLAY);
	BUTTON_NOT_VISIBLE(BUTTON_PREV);
	BUTTON_NOT_VISIBLE(BUTTON_NEXT);
	BUTTON_NOT_VISIBLE(BUTTON_STOP);
	BUTTON_NOT_VISIBLE(BUTTON_REPEAT);
	BUTTON_NOT_VISIBLE(BUTTON_RANDOM);
	BUTTON_NOT_VISIBLE(BUTTON_PL_PREV);
	BUTTON_NOT_VISIBLE(BUTTON_PL_NEXT);
	BUTTON_NOT_VISIBLE(BUTTON_PL_ADD);
	BUTTON_NOT_VISIBLE(BUTTON_PL_REPLACE);
	BUTTON_NOT_VISIBLE(BUTTON_PL_RELOAD);
	
	// Content
	switch (mode) {
		case MODE_WEATHER:
			weather_db_output();	
			BUTTON_SWITCH_ON(BUTTON_WEATHER);
			break;
		case MODE_MUSIC:
			mpd_music_output();
			BUTTON_SWITCH_ON(BUTTON_MUSIC);
			BUTTON_IS_VISIBLE(BUTTON_PLAY);
			BUTTON_IS_VISIBLE(BUTTON_PREV);
			BUTTON_IS_VISIBLE(BUTTON_NEXT);
			BUTTON_IS_VISIBLE(BUTTON_STOP);
			BUTTON_IS_VISIBLE(BUTTON_REPEAT);
			BUTTON_IS_VISIBLE(BUTTON_RANDOM);
			break;
		case MODE_QUEUE:
			mpd_queue_output();
			// Steuerungsbutton setzen
			if (playlists != NULL) {
				BUTTON_IS_VISIBLE(BUTTON_PL_ADD);
				BUTTON_IS_VISIBLE(BUTTON_PL_REPLACE);
				if (playlists->prev != NULL) {
					BUTTON_IS_VISIBLE(BUTTON_PL_PREV);
				}
				if (playlists->next != NULL) {
					BUTTON_IS_VISIBLE(BUTTON_PL_NEXT);
				}
			}
			BUTTON_IS_VISIBLE(BUTTON_PL_RELOAD);
			BUTTON_SWITCH_ON(BUTTON_QUEUE);
			break;
		default:
			break;	
		}				
	// Button
	for (i=0; i<NR_BUTTONS; i++) draw_button(&buttons[i]);
}

// ********************************************
void sec_beat(int signal) {
	// Uhrzeit
	now = time (NULL);
	now_tm	= localtime (&now);
	sprintf(date_time, "%02d.%02d.%04d; %02d:%02d:%02d", 
			now_tm->tm_mday, now_tm->tm_mon+1, now_tm->tm_year+1900,
			now_tm->tm_hour, now_tm->tm_min, now_tm->tm_sec);
	// jede 2.Minute Wetterdaten holen
	if ((now_tm->tm_sec==0)&&((now_tm->tm_min%2)==0)) weather_db_read();
	// MPD-Status-Abfrage anstossen
	if (mode==MODE_MUSIC) mpd_status_update(mpd);
	// Bildschirm neu aufbauen
	refresh_screen();
	// Display-Backlight steuern...
	if (backlight_timer) {
		if (!mpd_status_play) backlight_timer--;	
	} else {
		if (backlight_on) {
			backlight_on = 0;
			BACKLIGHT_SWITCH_OFF;
			mode = MODE_WEATHER;
		}
	}	
	// ...in einer Sekunde wieder...
	alarm(1);
} 


// ********************************************
// ********************************************
// ********************************************
int main(void)
{

	struct tsdev *ts;
	char *tsdevice=NULL;
	int i;

	signal(SIGSEGV, sig);
	signal(SIGINT,  sig);
	signal(SIGTERM, sig);
	
	// Display-Backlight anschalten
	BACKLIGHT_SWITCH_ON;
	
	// Framebuffer
	if (open_framebuffer(&xres,&yres)) {
		exit(1);	
	}

	// Touchsreen	
	if( (tsdevice = getenv("TSLIB_TSDEVICE")) != NULL ) {
		ts = ts_open(tsdevice,0);
	} else {
		// bei Start via inittab gibt es kein Env., deshalb hier
		// das korrekte Device eintragen, sonst funktioniert der
		// Zugriff auf Touch nicht...!
//		if (!(ts = ts_open("/dev/input/event0", 0)))
		if (!(ts = ts_open("/dev/input/touchscreen", 0)))
			ts = ts_open("/dev/touchscreen/ucb1x00", 0);
	}
	if (!ts) {
		perror("ts_open");
		exit(1);
	}
	if (ts_config(ts)) {
		perror("ts_config");
		exit(1);
	}
	
	// Verbindung zum MPD-Dienst oeffnen
#ifdef MPD_DEBUG
	debug_set_level(DEBUG_WARNING);
#endif
	mpd_init();
	mpd_lsplaylists(mpd);
	
	// Daten sammeln und das erste Mal entsprechend anzeigen
	weather_db_read();
	refresh_screen();
	
	// Sekundentimer initialisieren (fast so, wie bei MCs :-))
	signal(SIGALRM, sec_beat);
	alarm(1); 

	// Endlosschleife...
	while (1) {
		struct ts_sample samp;
		int ret;
		
		// Touchscreen?
		ret = ts_read(ts, &samp, 1);
		if (ret < 0) {
			perror("ts_read");
			exit(1);
		}
		if (ret != 1) continue;
		
		backlight_timer = BACKLIGHT_TIME;
		if (!backlight_on) {
			// wenn Backlight aus --> wieder anschalten, wenn Touch
			backlight_on = 1;
			BACKLIGHT_SWITCH_ON;
		} else {
			// ansonsten schauen, welcher Button	
			for (i = 0; i < NR_BUTTONS; i++)
				if (button_handle(&buttons[i], &samp)) {
					// welcher Button wurde beruehrt
					switch (i) {
					case BUTTON_EXIT:
						mpd_free(mpd);
						close_framebuffer();
						SHUTDOWN;
						//exit(0);
						break;
					case BUTTON_WEATHER:
						mode = MODE_WEATHER;
						refresh_screen();
						break;
					case BUTTON_MUSIC:	
						mode = MODE_MUSIC;
						refresh_screen();
						break;
					case BUTTON_QUEUE:	
						if (mode != MODE_QUEUE) {
							mode = MODE_QUEUE;
							// Playlisten 
							//mpd_lsplaylists(mpd);
						}
						refresh_screen();
						break;
					case BUTTON_PLAY:
						switch(mpd_player_get_state(mpd)) {
							case MPD_PLAYER_PLAY:
								mpd_player_pause(mpd);
								break;
							case MPD_PLAYER_PAUSE:
							case MPD_PLAYER_STOP:
								mpd_player_play(mpd);
								break;
						}
						break;
					case BUTTON_PREV:
						mpd_player_prev(mpd);	
						break;
					case BUTTON_NEXT:
						mpd_player_next(mpd);	
						break;
					case BUTTON_STOP:	
						mpd_player_stop(mpd);
						break;
					case BUTTON_REPEAT:	
						mpd_player_set_repeat(mpd, !mpd_player_get_repeat(mpd));
						break;
					case BUTTON_RANDOM:	
						mpd_player_set_random(mpd, !mpd_player_get_random(mpd));
						break;
					case BUTTON_PL_PREV:
						if (playlists->prev!=NULL) playlists=playlists->prev;
						refresh_screen();	
						break;
					case BUTTON_PL_NEXT:	
						if (playlists->next!=NULL) playlists=playlists->next;
						refresh_screen();	
						break;
					case BUTTON_PL_ADD:	
						mpd_playlist_queue_load(mpd, playlists->name);
						mpd_playlist_queue_commit(mpd);
						switch(mpd_player_get_state(mpd)) {
							case MPD_PLAYER_PAUSE:
							case MPD_PLAYER_STOP:
								mpd_player_play(mpd);
								break;
						}
						break;
					case BUTTON_PL_REPLACE:	
						mpd_playlist_clear(mpd);
						mpd_playlist_queue_load(mpd, playlists->name);
						mpd_playlist_queue_commit(mpd);
						switch(mpd_player_get_state(mpd)) {
							case MPD_PLAYER_PAUSE:
							case MPD_PLAYER_STOP:
								mpd_player_play(mpd);
								break;
						}
						break;
					case BUTTON_PL_RELOAD:	
						mpd_lsplaylists(mpd);
						break;
					}
				}
		}
	}	
	mpd_free(mpd);
	close_framebuffer();
}


